@@ -28,6 +28,29 @@
 #define strcasecmp _stricmp
 #endif
 
+/*
+ * filters mutually exclusive with file/translate/process filters
+ */
+static char *lzremoval(char *s)
+{
+char *p = s;
+
+if(*p)
+      	{
+	while((*p=='0') && *(p+1))
+                {
+                p++;
+                }
+        }
+
+if(p != s)
+	{
+	memmove(s, p, strlen(p) + 1);
+        }
+
+return(s);
+}
+
 /*
  * file/translate/process filters
  */
@@ -175,16 +198,17 @@ return(s);
 
 
 /*
- * convert binary <=> gray/popcnt in place
+ * convert binary <=> gray/ffo/popcnt in place
  */
 #define cvt_gray(f,p,n) \
 do { \
-if((f)&(TR_GRAYMASK|TR_POPCNT)) \
-	{ \
-	if((f)&TR_BINGRAY) { convert_bingray((p),(n)); } \
-	if((f)&TR_GRAYBIN) { convert_graybin((p),(n)); } \
-	if((f)&TR_POPCNT)  { convert_popcnt((p),(n)); } \
-	} \
+if((f)&(TR_GRAYMASK|TR_POPCNT|TR_FFO)) \
+        { \
+        if((f)&TR_BINGRAY) { convert_bingray((p),(n)); } \
+        if((f)&TR_GRAYBIN) { convert_graybin((p),(n)); } \
+	if((f)&TR_FFO)     { convert_ffo((p),(n)); } \
+        if((f)&TR_POPCNT)  { convert_popcnt((p),(n)); } \
+        } \
 } while(0)
 
 
@@ -314,6 +338,40 @@ for(i=nbits-1;i>=0;i--) /* always requires less number of bits */
 }
 
 
+static void convert_ffo(char *pnt, int nbits)
+{
+int i;
+int ffo = -1;
+
+for(i=(nbits-1);i>=0;i--)
+        {
+        char ch = pnt[i];
+
+        if((ch == AN_1) || (ch == AN_H))
+                {
+                ffo = (nbits-1) - i;
+                break;
+                }
+        }
+
+if(ffo >= 0)
+        {
+        for(i=nbits-1;i>=0;i--) /* always requires less number of bits */
+                {
+                pnt[i] = (ffo & 1) ? AN_1 : AN_0;
+                ffo >>= 1;
+                }
+        }
+	else
+	{
+	for(i=nbits-1;i>=0;i--) /* always requires less number of bits */
+                {
+                pnt[i] = AN_X;
+                }
+        }
+}
+
+
 static void dpr_e16(char *str, double d)
 {
 char *buf16;
@@ -392,7 +450,7 @@ static void cvt_fpsudec(Trptr t, TimeType val, char *os, int len)
 {
 int shamt = t->t_fpdecshift;
 UTimeType lpart = ((UTimeType)val) >> shamt;
-TimeType rmsk = (1 << shamt);
+TimeType rmsk = (ULLDescriptor(1) << shamt);
 TimeType rbit = (val & (rmsk-1));
 double rfrac;
 char dbuf[32];
@@ -613,7 +671,7 @@ if(flags&TR_ASCII)
 	if(GLOBALS->show_base) { *(pnt++)='"'; }
 	*(pnt)=0x00; /* scan build : remove dead increment */
 	}
-else if((flags&TR_HEX)||((flags&(TR_DEC|TR_SIGNED))&&(nbits>64)&&(!(flags&TR_POPCNT))))
+else if((flags&TR_HEX)||((flags&(TR_DEC|TR_SIGNED))&&(nbits>64)&&(!(flags&(TR_POPCNT|TR_FFO)))))
 	{
 	char *parse;
 
@@ -1043,25 +1101,29 @@ if(t && (t->flags & TR_REAL2BITS) && d) /* "real2bits" also allows other filters
 		}
 	}
 
-if(!(t->f_filter|t->p_filter|t->e_filter))
-        {
+if(t)
+	{
+	if(!(t->f_filter|t->p_filter|t->e_filter))
+	        {
+		if(GLOBALS->lz_removal) lzremoval(rv);
+		}
+	        else
+	        {
+	        if(t->e_filter)
+	                {
+	                rv = edofilter(t, rv);
+	                }
+	        else
+	        if(t->f_filter)
+	                {
+	                rv = dofilter(t, rv);
+	                }
+	                else
+	                {
+	                rv = pdofilter(t, rv);
+	                }
+	        }
 	}
-        else
-        {
-        if(t->e_filter)
-                {
-                rv = edofilter(t, rv);
-                }
-        else
-        if(t->f_filter)
-                {
-                rv = dofilter(t, rv);
-                }
-                else
-                {
-                rv = pdofilter(t, rv);
-                }
-        }
 
 return(rv);
 }
@@ -1382,7 +1444,7 @@ if(flags&TR_ASCII)
 	if(GLOBALS->show_base) { *(pnt++)='"'; }
 	*(pnt)=0x00; /* scan build : remove dead increment */
 	}
-else if((flags&TR_HEX)||((flags&(TR_DEC|TR_SIGNED))&&(nbits>64)&&(!(flags&TR_POPCNT))))
+else if((flags&TR_HEX)||((flags&(TR_DEC|TR_SIGNED))&&(nbits>64)&&(!(flags&(TR_POPCNT|TR_FFO)))))
 	{
 	char *parse;
 
@@ -1769,6 +1831,7 @@ char *s = convert_ascii_vec_2(t, vec);
 
 if(!(t->f_filter|t->p_filter|t->e_filter))
 	{
+	if(GLOBALS->lz_removal) lzremoval(s);
 	}
 	else
 	{
@@ -1819,6 +1882,7 @@ if((!t->t_filter_converted) && (!(v->flags & HIST_STRING)))
 
 if(!(t->f_filter|t->p_filter|t->e_filter))
 	{
+	if(GLOBALS->lz_removal) lzremoval(s);
 	}
 	else
 	{
@@ -2080,62 +2144,97 @@ if(flags&TR_REVERSE)
 	for(i=0;i<3;i++) *(fwdpnt2++)=xfwd[0];
 	}
 
-
-if(flags&TR_SIGNED)
+if(flags&TR_REAL)
 	{
-	char *parse;
-	TimeType val = 0;
-	unsigned char fail=0;
-
-	parse=newbuff+3;
-	cvt_gray(flags,parse,nbits);
-
-	if((parse[0]==AN_1)||(parse[0]==AN_H))
-		{ val = LLDescriptor(-1); }
-	else
-	if((parse[0]==AN_0)||(parse[0]==AN_L))
-		{ val = LLDescriptor(0); }
-	else
-		{ fail = 1; }
-
-	if(!fail)
-	for(i=1;i<nbits;i++)
+	if((nbits==64) || (nbits == 32)) /* fail (NaN) otherwise */
 		{
-		val<<=1;
+		char *parse;
+		UTimeType val=0;
+		unsigned char fail=0;
+		uint32_t val_32;
 
-		if((parse[i]==AN_1)||(parse[i]==AN_H)) {  val|=LLDescriptor(1); }
-		else if((parse[i]!=AN_0)&&(parse[i]!=AN_L)) { fail=1; break; }
-		}
+		parse=newbuff+3;
+		for(i=0;i<nbits;i++)
+			{
+			val<<=1;
 
-	if(!fail)
-		{
-		retval = val;
+		        if((parse[i]==AN_1)||(parse[i]==AN_H)) {  val|=LLDescriptor(1); }
+		        else if((parse[i]!=AN_0)&&(parse[i]!=AN_L)) { fail=1; break; }
+			}
+
+                if(!fail)
+                        {
+                        if(nbits==64)
+                                {
+                                memcpy(&retval, &val, sizeof(double)); /* otherwise strict-aliasing rules problem if retval = *(double *)&val; */
+                                }
+                                else
+                                {
+                                float f;
+                                val_32 = val;
+                                memcpy(&f, &val_32, sizeof(float)); /* otherwise strict-aliasing rules problem if retval = *(double *)&val; */
+                                retval = (double)f;
+                                }
+                        }
 		}
 	}
-else	/* decimal when all else fails */
-	{
-	char *parse;
-	UTimeType val=0;
-	unsigned char fail=0;
-
-	parse=newbuff+3;
-	cvt_gray(flags,parse,nbits);
-
-	for(i=0;i<nbits;i++)
+    else
+    {
+    if(flags&TR_SIGNED)
 		{
-		val<<=1;
+		char *parse;
+		TimeType val = 0;
+		unsigned char fail=0;
 
-		if((parse[i]==AN_1)||(parse[i]==AN_H)) {  val|=LLDescriptor(1); }
-		else if((parse[i]!=AN_0)&&(parse[i]!=AN_L)) { fail=1; break; }
+		parse=newbuff+3;
+		cvt_gray(flags,parse,nbits);
+
+		if((parse[0]==AN_1)||(parse[0]==AN_H))
+			{ val = LLDescriptor(-1); }
+		else
+		if((parse[0]==AN_0)||(parse[0]==AN_L))
+			{ val = LLDescriptor(0); }
+		else
+			{ fail = 1; }
+
+		if(!fail)
+		for(i=1;i<nbits;i++)
+			{
+			val<<=1;
+	
+			if((parse[i]==AN_1)||(parse[i]==AN_H)) {  val|=LLDescriptor(1); }
+			else if((parse[i]!=AN_0)&&(parse[i]!=AN_L)) { fail=1; break; }
+			}
+
+		if(!fail)
+			{
+			retval = val;
+			}
 		}
-
-	if(!fail)
+	else	/* decimal when all else fails */
 		{
-		retval = val;
+		char *parse;
+		UTimeType val=0;
+		unsigned char fail=0;
+	
+		parse=newbuff+3;
+		cvt_gray(flags,parse,nbits);
+	
+		for(i=0;i<nbits;i++)
+			{
+			val<<=1;
+	
+			if((parse[i]==AN_1)||(parse[i]==AN_H)) {  val|=LLDescriptor(1); }
+			else if((parse[i]!=AN_0)&&(parse[i]!=AN_L)) { fail=1; break; }
+			}
+	
+		if(!fail)
+			{
+			retval = val;
+			}
 		}
 	}
 
 free_2(newbuff);
 return(retval);
 }
-
