@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Tony Bybell 2009-2018.
+ * Copyright (c) Tony Bybell 2009-2025.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,6 +34,12 @@
 
 #define FST_RDLOAD "FSTLOAD | "
 
+/* 
+ * pseudo 2D arrays identified as broken in development version testing: fixed in LTS
+ * but removing via ifdef for now as this needs more testing and isn't likely useful
+ */
+#define FST_LEN_SUBST_IS_BROKEN
+
 /******************************************************************/
 
 /*
@@ -217,13 +223,14 @@ while((h = fstReaderIterateHier(xc)))
 			pntd = s;
 
 			int esc_sm = 0;
+                        int num_col = 0;
 			while(*pnts)
 				{
 				if(*pnts == '\\') esc_sm = 1;
 				if(*pnts != ' ')
 					{
 					if(*pnts == '[' && !esc_sm) { lb_last = pntd; col_last = NULL; rb_last = NULL; }
-					else if(*pnts == ':' && lb_last != NULL && col_last == NULL && rb_last == NULL && !esc_sm) { col_last = pntd; }
+					else if(*pnts == ':' && lb_last != NULL && col_last == NULL && rb_last == NULL && !esc_sm) { col_last = pntd; num_col++; }
 					else if(*pnts == ']' && lb_last != NULL && rb_last == NULL && !esc_sm) { rb_last = pntd; }
 					else if(lb_last != NULL && rb_last == NULL && (isdigit((int)(unsigned char)*pnts) || (*pnts == '-')) && !esc_sm) { }
 					else { lb_last = NULL; col_last = NULL; rb_last = NULL; }
@@ -297,6 +304,24 @@ while((h = fstReaderIterateHier(xc)))
 						}
 					*lsb = acc * sgnb;
 					}
+
+#ifndef FST_LEN_SUBST_IS_BROKEN
+				if(num_col < 2) /* 2d array processing if number of colons >= 2 */
+#endif
+					{
+					unsigned int abslen = (*msb >= *lsb) ? (*msb - *lsb + 1) : (*lsb - *msb + 1);
+					if((h->u.var.length != abslen) && (h->u.var.length))
+						{
+						if (*msb >= *lsb)
+							{
+							*msb = *lsb + h->u.var.length - 1;
+							}
+							else
+							{
+							*lsb = *msb + h->u.var.length - 1;
+							}
+						}
+					}
 				}
 
 			if(lb_last)
@@ -707,7 +732,9 @@ for(i=0;i<GLOBALS->numfacs;i++)
 	unsigned char nvt, nvd, ndt;
 	int longest_nam_candidate = 0;
 	char *fnam;
+#ifndef FST_LEN_SUBST_IS_BROKEN
 	int len_subst = 0;
+#endif
 
 	h = extractNextVar(GLOBALS->fst_fst_c_1, &msb, &lsb, &nnam, &name_len, &nnam_max);
 	if(!h)
@@ -769,6 +796,7 @@ for(i=0;i<GLOBALS->numfacs;i++)
 		}
 		else
 		{
+#ifndef FST_LEN_SUBST_IS_BROKEN
 		unsigned int abslen = (msb >= lsb) ? (msb - lsb + 1) : (lsb - msb + 1);
 
 		if((h->u.var.length > abslen) && !(h->u.var.length % abslen)) /* check if 2d array */
@@ -776,6 +804,7 @@ for(i=0;i<GLOBALS->numfacs;i++)
 			/* printf("h->u.var.length: %d, abslen: %d '%s'\n", h->u.var.length, abslen, fnam); */
 			len_subst = 1;
 			}
+#endif
 
 		node_block[i].msi = msb;
 		node_block[i].lsi = lsb;
@@ -905,12 +934,6 @@ for(i=0;i<GLOBALS->numfacs;i++)
 		{
 		int len=sprintf_2_sdd(buf, f_name[(i)&F_NAME_MODULUS],node_block[i].msi, node_block[i].lsi);
 
-		if(len_subst) /* preserve 2d in name, but make 1d internally */
-			{
-			node_block[i].msi = h->u.var.length-1;
-			node_block[i].lsi = 0;
-			}
-
 		longest_nam_candidate = len;
 
 		if(!GLOBALS->do_hier_compress)
@@ -943,6 +966,14 @@ for(i=0;i<GLOBALS->numfacs;i++)
 			len = sprintf_2_sdd(buf, nnam,node_block[i].msi, node_block[i].lsi);
 			fst_append_graft_chain(len, buf, i, npar);
 			}
+
+#ifndef FST_LEN_SUBST_IS_BROKEN
+		if(len_subst) /* preserve 2d in name, but make 1d internally */
+			{
+			node_block[i].msi = h->u.var.length-1;
+			node_block[i].lsi = 0;
+			}
+#endif
 		}
 	else
 		{
@@ -1535,7 +1566,7 @@ else	/* string */
 
 	if((l2e->histent_curr)&&(l2e->histent_curr->v.h_vector)) /* remove duplicate values */
 		{
-		if((!strcmp(l2e->histent_curr->v.h_vector, (const char *)value)) && (!GLOBALS->vcd_preserve_glitches))
+		if((!strcmp(l2e->histent_curr->v.h_vector, s)) && (!GLOBALS->vcd_preserve_glitches))
 			{
 			free_2(s);
 			return;
