@@ -1076,6 +1076,7 @@ NULL, /* vcd_jmp_buf */
 -1, /* vcd_warning_filesize 472 */
 1, /* autocoalesce 473 */
 0, /* autocoalesce_reversal */
+0, /* mti_realparam_fix */
 -1, /* vcd_explicit_zero_subscripts 474 */
 0, /* convert_to_reals 475 */
 1, /* atomic_vectors 476 */
@@ -1341,6 +1342,9 @@ NULL, /* swipe_init_time */
 -1, /* use_gestures */
 FALSE, /*use_dark */
 FALSE, /*save_on_exit */
+#ifdef GDK_WINDOWING_WAYLAND
+0, /* wayland_marker_timer_hack */
+#endif
 
 
 /*
@@ -1834,6 +1838,7 @@ void reload_into_new_context_2(void)
  new_globals->autoname_bundles = GLOBALS->autoname_bundles;
  new_globals->autocoalesce = GLOBALS->autocoalesce;
  new_globals->autocoalesce_reversal = GLOBALS->autocoalesce_reversal;
+ new_globals->mti_realparam_fix = GLOBALS->mti_realparam_fix;
  new_globals->constant_marker_update = GLOBALS->constant_marker_update;
  new_globals->convert_to_reals = GLOBALS->convert_to_reals;
  new_globals->disable_mouseover = GLOBALS->disable_mouseover;
@@ -2818,6 +2823,7 @@ switch(type)
 							GLOBALS->lxt_clock_compress_to_z = g_old->lxt_clock_compress_to_z;
 							GLOBALS->autoname_bundles = g_old->autoname_bundles;
 							GLOBALS->autocoalesce_reversal = g_old->autocoalesce_reversal;
+							GLOBALS->mti_realparam_fix = g_old->mti_realparam_fix;
 							GLOBALS->autocoalesce = g_old->autocoalesce;
 							GLOBALS->hier_grouping = g_old->hier_grouping;
 							GLOBALS->wave_scrolling = g_old->wave_scrolling;
