@@ -1099,6 +1099,7 @@ jmp_buf *vcd_jmp_buf; /* from vcd.c */
 int vcd_warning_filesize; /* from vcd.c 502 */
 char autocoalesce; /* from vcd.c 503 */
 char autocoalesce_reversal; /* from vcd.c 504 */
+char mti_realparam_fix; 
 int vcd_explicit_zero_subscripts; /* from vcd.c 505 */
 char convert_to_reals; /* from vcd.c 506 */
 char atomic_vectors; /* from vcd.c 507 */
@@ -1364,6 +1365,9 @@ char wavearea_drag_active;
 char use_gestures;
 gboolean use_dark;
 gboolean save_on_exit;
+#ifdef GDK_WINDOWING_WAYLAND
+int wayland_marker_timer_hack;
+#endif
 
 
 /*
