@@ -167,6 +167,8 @@ WV_MENU_G2B,
 WV_MENU_GBNONE,
 WV_MENU_POPON,
 WV_MENU_POPOFF,
+WV_MENU_FFOON,
+WV_MENU_FFOOFF,
 WV_MENU_FPSHIFTON,
 WV_MENU_FPSHIFTOFF,
 WV_MENU_FPSHIFTVAL,
@@ -264,6 +266,7 @@ WV_MENU_VSG,
 WV_MENU_SEP9,
 WV_MENU_SHW,
 WV_MENU_FILL1,
+WV_MENU_LZ_REMOVAL,
 WV_MENU_SEP9B,
 WV_MENU_VSMO,
 #ifdef WAVE_USE_GTK2
@@ -326,6 +329,7 @@ void menu_new_viewer(gpointer null_data, guint callback_action, GtkWidget *widge
 void menu_write_vcd_file(gpointer null_data, guint callback_action, GtkWidget *widget);
 void menu_write_lxt_file(gpointer null_data, guint callback_action, GtkWidget *widget);
 void menu_print(gpointer null_data, guint callback_action, GtkWidget *widget);
+void menu_write_save_cleanup(GtkWidget *widget, gpointer data);
 void menu_read_save_file(gpointer null_data, guint callback_action, GtkWidget *widget);
 void menu_write_save_file(gpointer null_data, guint callback_action, GtkWidget *widget);
 void menu_write_save_file_as(gpointer null_data, guint callback_action, GtkWidget *widget);
