@@ -133,19 +133,18 @@ static void load_proc_filter(int which, char *name)
 #if !defined __MINGW32__ && !defined _MSC_VER
   cmd = (char *)malloc_2(strlen(exec_name)+6+1);
   sprintf(cmd, "which %s", exec_name);
-  stream = popen(cmd, "r");
+  stream = popen_san(cmd, "r");
 
   result = fscanf(stream, "%s", abs_path);
+  pclose(stream);
+  free_2(cmd);
 
   if((strlen(abs_path) == 0)||(!result))
     {
       status_text("Could not find filter process!\n");
-      pclose(stream); /* cppcheck */
       return;
     }
 
-  pclose(stream);
-  free_2(cmd);
 #else
   strcpy(abs_path, exec_name);
 #endif
