@@ -453,7 +453,7 @@ return(T_UNKNOWN_KEY);
 }
 
 
-static int get_vartoken_patched(int match_kw)
+static int get_vartoken_patched(int ignore_colon, int match_kw)
 {
 int ch;
 int len=0;
@@ -475,7 +475,7 @@ if(!GLOBALS->var_prevch_vcd_c_1)
 	}
 
 if(ch=='[') return(V_LB);
-if(ch==':') return(V_COLON);
+if(ch==':' && !ignore_colon) return(V_COLON);
 if(ch==']') return(V_RB);
 
 for(GLOBALS->yytext_vcd_c_1[len++]=ch;;GLOBALS->yytext_vcd_c_1[len++]=ch)
@@ -486,7 +486,7 @@ for(GLOBALS->yytext_vcd_c_1[len++]=ch;;GLOBALS->yytext_vcd_c_1[len++]=ch)
 		}
 	ch=getch_patched();
 	if(ch<0) { free_2(GLOBALS->varsplit_vcd_c_1); GLOBALS->varsplit_vcd_c_1=NULL; break; }
-	if((ch==':')||(ch==']'))
+	if((ch==':' && !ignore_colon)||(ch==']'))
 		{
 		GLOBALS->var_prevch_vcd_c_1=ch;
 		break;
@@ -509,14 +509,14 @@ if(ch<0) { free_2(GLOBALS->varsplit_vcd_c_1); GLOBALS->varsplit_vcd_c_1=NULL; }
 return(V_STRING);
 }
 
-static int get_vartoken(int match_kw)
+static int get_vartoken(int ignore_colon, int match_kw)
 {
 int ch;
 int len=0;
 
 if(GLOBALS->varsplit_vcd_c_1)
 	{
-	int rc=get_vartoken_patched(match_kw);
+	int rc=get_vartoken_patched(ignore_colon, match_kw);
 	if(rc!=V_END) return(rc);
 	GLOBALS->var_prevch_vcd_c_1=0;
 	}
@@ -538,7 +538,7 @@ if(!GLOBALS->var_prevch_vcd_c_1)
 	}
 
 if(ch=='[') return(V_LB);
-if(ch==':') return(V_COLON);
+if(ch==':' && !ignore_colon) return(V_COLON);
 if(ch==']') return(V_RB);
 
 if(ch=='#')     /* for MTI System Verilog '$var reg 64 >w #implicit-var###VarElem:ram_di[0.0] [63:0] $end' style declarations */
@@ -580,7 +580,7 @@ for(GLOBALS->yytext_vcd_c_1[len++]=ch;;GLOBALS->yytext_vcd_c_1[len++]=ch)
 		GLOBALS->varsplit_vcd_c_1=GLOBALS->yytext_vcd_c_1+len;		/* keep looping so we get the *last* one */
 		}
 	else
-	if(((ch==':')||(ch==']'))&&(!GLOBALS->varsplit_vcd_c_1)&&(GLOBALS->yytext_vcd_c_1[0]!='\\'))
+	if(((ch==':' && !ignore_colon)||(ch==']'))&&(!GLOBALS->varsplit_vcd_c_1)&&(GLOBALS->yytext_vcd_c_1[0]!='\\'))
 		{
 		GLOBALS->var_prevch_vcd_c_1=ch;
 		break;
@@ -683,6 +683,11 @@ for(;;)
                 GLOBALS->autocoalesce = 0;
 		rc = 1;
                 }
+        else if(strstr(GLOBALS->yytext_vcd_c_1, "Questa") || strstr(GLOBALS->yytext_vcd_c_1, "ModelSim")) /* realparam fix only is for MTI, conflicts with Vivado */
+                {
+                GLOBALS->mti_realparam_fix = 1;
+                rc = 1;
+                }
 	}
 if(hdr) { DEBUG(fprintf(stderr,"\n")); }
 return(rc);
@@ -1002,6 +1007,15 @@ switch(GLOBALS->yytext_vcd_c_1[0])
 			}
 			else
 			{
+                        if(v->vartype == V_PARAMETER) /* github #446: size 0 on parameter with type declared wrong */
+                                {
+                                v->vartype = V_REAL;    /* override any data we parsed in for $var declaration */
+                                v->size=1;
+                                v->msi=v->lsi=0;
+
+                                fprintf(stderr, "GTKWAVE | Warning: symbol '%s' changing datatype from parameter to real.\n", v->name);
+                                }
+
 			add_histent(GLOBALS->current_time_vcd_c_1, v->narray[0],'g',1,(char *)d);
 			}
 
@@ -1075,6 +1089,8 @@ static void vcd_parse(void)
 int tok;
 unsigned char ttype;
 int disable_autocoalesce = 0;
+int colon_seen = 0;
+int num_seen = 0;
 
 for(;;)
 	{
@@ -1273,7 +1289,7 @@ for(;;)
 				free_2(GLOBALS->varsplit_vcd_c_1);
 				GLOBALS->varsplit_vcd_c_1=NULL;
 				}
-			vtok=get_vartoken(1);
+			vtok=get_vartoken(0, 1);
 			if(vtok>V_STRINGTYPE) goto bail;
 
 			v=(struct vcdsymbol *)calloc_2(1,sizeof(struct vcdsymbol));
@@ -1282,7 +1298,7 @@ for(;;)
 
 			if(vtok==V_PORT)
 				{
-				vtok=get_vartoken(1);
+				vtok=get_vartoken(0, 1);
 				if(vtok==V_STRING)
 					{
 					v->size=atoi_64(GLOBALS->yytext_vcd_c_1);
@@ -1291,11 +1307,11 @@ for(;;)
 					else
 					if(vtok==V_LB)
 					{
-					vtok=get_vartoken(1);
+					vtok=get_vartoken(0, 1);
 					if(vtok==V_END) goto err;
 					if(vtok!=V_STRING) goto err;
 					v->msi=atoi_64(GLOBALS->yytext_vcd_c_1);
-					vtok=get_vartoken(0);
+					vtok=get_vartoken(0, 0);
 					if(vtok==V_RB)
 						{
 						v->lsi=v->msi;
@@ -1304,10 +1320,10 @@ for(;;)
 						else
 						{
 						if(vtok!=V_COLON) goto err;
-						vtok=get_vartoken(0);
+						vtok=get_vartoken(0, 0);
 						if(vtok!=V_STRING) goto err;
 						v->lsi=atoi_64(GLOBALS->yytext_vcd_c_1);
-						vtok=get_vartoken(0);
+						vtok=get_vartoken(0, 0);
 						if(vtok!=V_RB) goto err;
 
 						if(v->msi>v->lsi)
@@ -1345,7 +1361,7 @@ for(;;)
                                 if(v->nid < GLOBALS->vcd_minid_vcd_c_1) GLOBALS->vcd_minid_vcd_c_1 = v->nid;
                                 if(v->nid > GLOBALS->vcd_maxid_vcd_c_1) GLOBALS->vcd_maxid_vcd_c_1 = v->nid;
 
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(1, 0);
 				if(vtok!=V_STRING) goto err;
 				if(GLOBALS->slisthier_len)
 					{
@@ -1392,7 +1408,7 @@ for(;;)
 
                                 if(GLOBALS->pv_vcd_c_1)
                                         {
-                                        if(!strcmp(GLOBALS->pv_vcd_c_1->name,v->name) && !disable_autocoalesce && (!strchr(v->name, '\\')))
+                                        if(!strcmp(GLOBALS->pv_vcd_c_1->name,v->name) && !disable_autocoalesce && (!strchr(v->name, '\\')) && (v->lsi != GLOBALS->pv_vcd_c_1->lsi))
                                                 {
                                                 GLOBALS->pv_vcd_c_1->chain=v;
                                                 v->root=GLOBALS->rootv_vcd_c_1;
@@ -1411,7 +1427,7 @@ for(;;)
 				}
 				else	/* regular vcd var, not an evcd port var */
 				{
-				vtok=get_vartoken(1);
+				vtok=get_vartoken(0, 1);
 				if(vtok==V_END) goto err;
 				v->size=atoi_64(GLOBALS->yytext_vcd_c_1);
 				vtok=get_strtoken();
@@ -1437,7 +1453,7 @@ for(;;)
                                 if(v->nid < GLOBALS->vcd_minid_vcd_c_1) GLOBALS->vcd_minid_vcd_c_1 = v->nid;
                                 if(v->nid > GLOBALS->vcd_maxid_vcd_c_1) GLOBALS->vcd_maxid_vcd_c_1 = v->nid;
 
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(1, 0);
 				if(vtok!=V_STRING) goto err;
 				if(GLOBALS->slisthier_len)
 					{
@@ -1484,7 +1500,7 @@ for(;;)
 
                                 if(GLOBALS->pv_vcd_c_1)
                                         {
-                                        if(!strcmp(GLOBALS->pv_vcd_c_1->name,v->name))
+                                        if(!strcmp(GLOBALS->pv_vcd_c_1->name,v->name) && (v->lsi != GLOBALS->pv_vcd_c_1->lsi))
                                                 {
                                                 GLOBALS->pv_vcd_c_1->chain=v;
                                                 v->root=GLOBALS->rootv_vcd_c_1;
@@ -1501,23 +1517,27 @@ for(;;)
 					}
                                 GLOBALS->pv_vcd_c_1=v;
 
-				vtok=get_vartoken(1);
+				num_seen = 0;
+				vtok=get_vartoken(0, 1);
 				if(vtok==V_END) goto dumpv;
 				if(vtok!=V_LB) goto err;
-				vtok=get_vartoken(0);
+				colon_seen = 0;
+				vtok=get_vartoken(0, 0);
 				if(vtok!=V_STRING) goto err;
+				num_seen = 1;
 				v->msi=atoi_64(GLOBALS->yytext_vcd_c_1);
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(0, 0);
 				if(vtok==V_RB)
 					{
 					v->lsi=v->msi;
 					goto dumpv;
 					}
 				if(vtok!=V_COLON) goto err;
-				vtok=get_vartoken(0);
+				colon_seen = 1;
+				vtok=get_vartoken(0, 0);
 				if(vtok!=V_STRING) goto err;
 				v->lsi=atoi_64(GLOBALS->yytext_vcd_c_1);
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(0, 0);
 				if(vtok!=V_RB) goto err;
 				}
 
@@ -1528,7 +1548,14 @@ for(;;)
                                         {
 					if(v->vartype != V_STRINGTYPE)
 						{
-	                                        v->vartype = V_REAL;
+						if(!GLOBALS->mti_realparam_fix)
+							{
+							v->size = 1;
+							}
+						else
+							{
+		                                        v->vartype = V_REAL;
+							}
 						}
                                         }
                                         else
@@ -1537,6 +1564,47 @@ for(;;)
                                         }
 
                                 } /* MTI fix */
+                                else
+                                {
+                                unsigned int abslen = (v->msi >= v->lsi) ? (v->msi - v->lsi + 1) : (v->lsi - v->msi + 1);
+
+                                if(!colon_seen)
+                                        {
+                                        if(num_seen && (v->size > 1))
+                                                {
+                                                char buf[32];
+                                                char *sfix;
+                                                int vname_len = strlen(v->name);
+                                                int num_len = snprintf(buf, 32, "[%d]", v->lsi);
+
+                                                sfix = malloc_2(vname_len + num_len + 1);
+                                                memcpy(sfix, v->name, vname_len);
+                                                memcpy(sfix+vname_len, buf, num_len + 1);
+                                                free_2(v->name);
+                                                v->name = sfix;
+                                                }
+
+                                        if(v->size > 1)
+                                                {
+                                                v->lsi = 0;
+                                                v->msi = v->size - 1;
+                                                }
+                                        }
+                                        else
+                                        {
+                                        if((v->size != abslen) && (num_seen))
+                                                {
+                                                if(v->lsi < v->msi)
+                                                        {
+                                                        v->msi = v->lsi + v->size - 1;
+                                                        }
+                                                        else
+                                                        {
+                                                        v->lsi = v->msi + v->size - 1;
+                                                        }
+                                                }
+                                        }
+                                }
 
 			if((v->vartype==V_REAL)||(v->vartype==V_STRINGTYPE)||((GLOBALS->convert_to_reals)&&((v->vartype==V_INTEGER)||(v->vartype==V_PARAMETER))))
 				{
@@ -1836,7 +1904,7 @@ for(;;)
 
 /*******************************************************************************/
 
-void add_histent(TimeType tim, struct Node *n, char ch, int regadd, char *vector)
+static void add_histent(TimeType tim, struct Node *n, char ch, int regadd, char *vector)
 {
 struct HistEnt *he;
 char heval;
@@ -1923,6 +1991,14 @@ switch(ch)
 		{
 		if(regadd) { tim*=(GLOBALS->time_scale); }
 
+                if(
+		  (n->curr->flags != (HIST_STRING|HIST_REAL)) /* because n->curr->v.h_vector could be a double and not a pointer */
+                  	||(n->curr->v.h_vector&&vector&&(strcmp(n->curr->v.h_vector,vector)))
+                        ||(tim==GLOBALS->start_time_vcd_c_1)
+                        ||(!n->curr->v.h_vector)
+                        ||(GLOBALS->vcd_preserve_glitches)
+                        ) /* same region == go skip */
+                        {
 			if(n->curr->time>=tim) /* backtracking fix */
 				{
 				DEBUG(printf("Warning: String Glitch at time ["TTFormat"] Signal [%p].\n",
@@ -1948,6 +2024,12 @@ switch(ch)
 				n->curr=he;
 	                	GLOBALS->regions+=regadd;
 				}
+			}
+                        else
+                        {
+                        free_2(vector);
+                        }
+
 	       }
 	break;
 	}
