@@ -428,7 +428,7 @@ return(T_UNKNOWN_KEY);
 }
 
 
-static int get_vartoken_patched(int match_kw)
+static int get_vartoken_patched(int ignore_colon, int match_kw)
 {
 int ch;
 int len=0;
@@ -450,7 +450,7 @@ if(!GLOBALS->var_prevch_vcd_partial_c_2)
 	}
 
 if(ch=='[') return(V_LB);
-if(ch==':') return(V_COLON);
+if(ch==':' && !ignore_colon) return(V_COLON);
 if(ch==']') return(V_RB);
 
 for(GLOBALS->yytext_vcd_partial_c_2[len++]=ch;;GLOBALS->yytext_vcd_partial_c_2[len++]=ch)
@@ -461,7 +461,7 @@ for(GLOBALS->yytext_vcd_partial_c_2[len++]=ch;;GLOBALS->yytext_vcd_partial_c_2[l
 		}
 	ch=getch_patched();
 	if(ch<0) { free_2(GLOBALS->varsplit_vcd_partial_c_2); GLOBALS->varsplit_vcd_partial_c_2=NULL; break; }
-	if((ch==':')||(ch==']'))
+	if((ch==':' && !ignore_colon)||(ch==']'))
 		{
 		GLOBALS->var_prevch_vcd_partial_c_2=ch;
 		break;
@@ -484,14 +484,14 @@ if(ch<0) { free_2(GLOBALS->varsplit_vcd_partial_c_2); GLOBALS->varsplit_vcd_part
 return(V_STRING);
 }
 
-static int get_vartoken(int match_kw)
+static int get_vartoken(int ignore_colon, int match_kw)
 {
 int ch;
 int len=0;
 
 if(GLOBALS->varsplit_vcd_partial_c_2)
 	{
-	int rc=get_vartoken_patched(match_kw);
+	int rc=get_vartoken_patched(ignore_colon, match_kw);
 	if(rc!=V_END) return(rc);
 	GLOBALS->var_prevch_vcd_partial_c_2=0;
 	}
@@ -513,7 +513,7 @@ if(!GLOBALS->var_prevch_vcd_partial_c_2)
 	}
 
 if(ch=='[') return(V_LB);
-if(ch==':') return(V_COLON);
+if(ch==':' && !ignore_colon) return(V_COLON);
 if(ch==']') return(V_RB);
 
 if(ch=='#')     /* for MTI System Verilog '$var reg 64 >w #implicit-var###VarElem:ram_di[0.0] [63:0] $end' style declarations */
@@ -555,7 +555,7 @@ for(GLOBALS->yytext_vcd_partial_c_2[len++]=ch;;GLOBALS->yytext_vcd_partial_c_2[l
 		GLOBALS->varsplit_vcd_partial_c_2=GLOBALS->yytext_vcd_partial_c_2+len;		/* keep looping so we get the *last* one */
 		}
 	else
-	if(((ch==':')||(ch==']'))&&(!GLOBALS->varsplit_vcd_partial_c_2)&&(GLOBALS->yytext_vcd_partial_c_2[0]!='\\'))
+	if(((ch==':' && !ignore_colon)||(ch==']'))&&(!GLOBALS->varsplit_vcd_partial_c_2)&&(GLOBALS->yytext_vcd_partial_c_2[0]!='\\'))
 		{
 		GLOBALS->var_prevch_vcd_partial_c_2=ch;
 		break;
@@ -658,6 +658,11 @@ for(;;)
 		GLOBALS->autocoalesce = 0;
                 rc = 1;
                 }
+        else if(strstr(GLOBALS->yytext_vcd_partial_c_2, "Questa") || strstr(GLOBALS->yytext_vcd_partial_c_2, "ModelSim")) /* realparam fix only is for MTI, conflicts with Vivado */
+                {
+                GLOBALS->mti_realparam_fix = 1;
+                rc = 1;
+                }
 	}
 if(hdr) { DEBUG(fprintf(stderr,"\n")); }
 return(rc);
@@ -1023,6 +1028,8 @@ static void vcd_parse(void)
 int tok;
 unsigned char ttype;
 int disable_autocoalesce = 0;
+int colon_seen = 0;
+int num_seen = 0;
 
 for(;;)
 	{
@@ -1219,7 +1226,7 @@ for(;;)
 				free_2(GLOBALS->varsplit_vcd_partial_c_2);
 				GLOBALS->varsplit_vcd_partial_c_2=NULL;
 				}
-			vtok=get_vartoken(1);
+			vtok=get_vartoken(0, 1);
 			if(vtok>V_STRINGTYPE) goto bail;
 
 			v=(struct vcdsymbol *)calloc_2(1,sizeof(struct vcdsymbol));
@@ -1228,7 +1235,7 @@ for(;;)
 
 			if(vtok==V_PORT)
 				{
-				vtok=get_vartoken(1);
+				vtok=get_vartoken(0, 1);
 				if(vtok==V_STRING)
 					{
 					v->size=atoi_64(GLOBALS->yytext_vcd_partial_c_2);
@@ -1237,11 +1244,11 @@ for(;;)
 					else
 					if(vtok==V_LB)
 					{
-					vtok=get_vartoken(1);
+					vtok=get_vartoken(0, 1);
 					if(vtok==V_END) goto err;
 					if(vtok!=V_STRING) goto err;
 					v->msi=atoi_64(GLOBALS->yytext_vcd_partial_c_2);
-					vtok=get_vartoken(0);
+					vtok=get_vartoken(0, 0);
 					if(vtok==V_RB)
 						{
 						v->lsi=v->msi;
@@ -1250,10 +1257,10 @@ for(;;)
 						else
 						{
 						if(vtok!=V_COLON) goto err;
-						vtok=get_vartoken(0);
+						vtok=get_vartoken(0, 0);
 						if(vtok!=V_STRING) goto err;
 						v->lsi=atoi_64(GLOBALS->yytext_vcd_partial_c_2);
-						vtok=get_vartoken(0);
+						vtok=get_vartoken(0, 0);
 						if(vtok!=V_RB) goto err;
 
 						if(v->msi>v->lsi)
@@ -1291,7 +1298,7 @@ for(;;)
                                 if(v->nid < GLOBALS->vcd_minid_vcd_partial_c_2) GLOBALS->vcd_minid_vcd_partial_c_2 = v->nid;
                                 if(v->nid > GLOBALS->vcd_maxid_vcd_partial_c_2) GLOBALS->vcd_maxid_vcd_partial_c_2 = v->nid;
 
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(1, 0);
 				if(vtok!=V_STRING) goto err;
 				if(GLOBALS->slisthier_len)
 					{
@@ -1338,7 +1345,7 @@ for(;;)
 
                                 if(GLOBALS->pv_vcd_partial_c_2)
                                         {
-                                        if(!strcmp(GLOBALS->pv_vcd_partial_c_2->name,v->name) && !disable_autocoalesce && (!strchr(v->name, '\\')))
+                                        if(!strcmp(GLOBALS->pv_vcd_partial_c_2->name,v->name) && !disable_autocoalesce && (!strchr(v->name, '\\')) && (v->lsi != GLOBALS->pv_vcd_partial_c_2->lsi))
                                                 {
                                                 GLOBALS->pv_vcd_partial_c_2->chain=v;
                                                 v->root=GLOBALS->rootv_vcd_partial_c_2;
@@ -1357,7 +1364,7 @@ for(;;)
 				}
 				else	/* regular vcd var, not an evcd port var */
 				{
-				vtok=get_vartoken(1);
+				vtok=get_vartoken(0, 1);
 				if(vtok==V_END) goto err;
 				v->size=atoi_64(GLOBALS->yytext_vcd_partial_c_2);
 				vtok=get_strtoken();
@@ -1383,7 +1390,7 @@ for(;;)
                                 if(v->nid < GLOBALS->vcd_minid_vcd_partial_c_2) GLOBALS->vcd_minid_vcd_partial_c_2 = v->nid;
                                 if(v->nid > GLOBALS->vcd_maxid_vcd_partial_c_2) GLOBALS->vcd_maxid_vcd_partial_c_2 = v->nid;
 
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(1, 0);
 				if(vtok!=V_STRING) goto err;
 				if(GLOBALS->slisthier_len)
 					{
@@ -1430,7 +1437,7 @@ for(;;)
 
                                 if(GLOBALS->pv_vcd_partial_c_2)
                                         {
-                                        if(!strcmp(GLOBALS->pv_vcd_partial_c_2->name,v->name))
+                                        if(!strcmp(GLOBALS->pv_vcd_partial_c_2->name,v->name) && (v->lsi != GLOBALS->pv_vcd_partial_c_2->lsi))
                                                 {
                                                 GLOBALS->pv_vcd_partial_c_2->chain=v;
                                                 v->root=GLOBALS->rootv_vcd_partial_c_2;
@@ -1447,23 +1454,27 @@ for(;;)
 					}
                                 GLOBALS->pv_vcd_partial_c_2=v;
 
-				vtok=get_vartoken(1);
+				num_seen = 0;
+				vtok=get_vartoken(0, 1);
 				if(vtok==V_END) goto dumpv;
 				if(vtok!=V_LB) goto err;
-				vtok=get_vartoken(0);
+				colon_seen = 0;
+				vtok=get_vartoken(0, 0);
 				if(vtok!=V_STRING) goto err;
+				num_seen = 1;
 				v->msi=atoi_64(GLOBALS->yytext_vcd_partial_c_2);
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(0, 0);
 				if(vtok==V_RB)
 					{
 					v->lsi=v->msi;
 					goto dumpv;
 					}
 				if(vtok!=V_COLON) goto err;
-				vtok=get_vartoken(0);
+				colon_seen = 1;
+				vtok=get_vartoken(0, 0);
 				if(vtok!=V_STRING) goto err;
 				v->lsi=atoi_64(GLOBALS->yytext_vcd_partial_c_2);
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(0, 0);
 				if(vtok!=V_RB) goto err;
 				}
 
@@ -1474,7 +1485,14 @@ for(;;)
                                         {
 					if(v->vartype != V_STRINGTYPE)
 						{
-	                                        v->vartype = V_REAL;
+						if(!GLOBALS->mti_realparam_fix)
+							{
+							v->size = 1;
+							}
+						else
+							{
+	                                        	v->vartype = V_REAL;
+							}
 						}
                                         }
                                         else
@@ -1483,6 +1501,47 @@ for(;;)
                                         }
 
                                 } /* MTI fix */
+                                else
+                                {
+                                unsigned int abslen = (v->msi >= v->lsi) ? (v->msi - v->lsi + 1) : (v->lsi - v->msi + 1);
+
+                                if(!colon_seen)
+                                        {
+                                        if(num_seen && (v->size > 1))
+                                                {
+                                                char buf[32];
+                                                char *sfix;
+                                                int vname_len = strlen(v->name);
+                                                int num_len = snprintf(buf, 32, "[%d]", v->lsi);
+
+                                                sfix = malloc_2(vname_len + num_len + 1);
+                                                memcpy(sfix, v->name, vname_len);
+                                                memcpy(sfix+vname_len, buf, num_len + 1);
+                                                free_2(v->name);
+                                                v->name = sfix;
+                                                }
+
+                                        if(v->size > 1)
+                                                {
+                                                v->lsi = 0;
+                                                v->msi = v->size - 1;
+                                                }
+                                        }
+                                        else
+                                        {
+                                        if((v->size != abslen) && (num_seen))
+                                                {
+                                                if(v->lsi < v->msi)
+                                                        {
+                                                        v->msi = v->lsi + v->size - 1;
+                                                        }
+                                                        else
+                                                        {
+                                                        v->lsi = v->msi + v->size - 1;
+                                                        }
+                                                }
+                                        }
+                                }
 
 			if((v->vartype==V_REAL)||(v->vartype==V_STRINGTYPE)||((GLOBALS->convert_to_reals)&&((v->vartype==V_INTEGER)||(v->vartype==V_PARAMETER))))
 				{
