@@ -848,6 +848,7 @@ errno = 0;
 if(feof(GLOBALS->vcd_handle_vcd_recoder_c_2)) return(-1);
 
 GLOBALS->vcdbyteno_vcd_recoder_c_3+=(GLOBALS->vend_vcd_recoder_c_3-GLOBALS->vcdbuf_vcd_recoder_c_3);
+memset(GLOBALS->vcdbuf_vcd_recoder_c_3, 0, VCD_BSIZ);
 rd=fread(GLOBALS->vcdbuf_vcd_recoder_c_3, sizeof(char), VCD_BSIZ, GLOBALS->vcd_handle_vcd_recoder_c_2);
 GLOBALS->vend_vcd_recoder_c_3=(GLOBALS->vst_vcd_recoder_c_3=GLOBALS->vcdbuf_vcd_recoder_c_3)+rd;
 
@@ -864,13 +865,13 @@ return((int)(*GLOBALS->vst_vcd_recoder_c_3));
 static inline signed char getch(void) {
   signed char ch = (GLOBALS->vst_vcd_recoder_c_3!=GLOBALS->vend_vcd_recoder_c_3)?((int)(*GLOBALS->vst_vcd_recoder_c_3)):(getch_fetch());
   GLOBALS->vst_vcd_recoder_c_3++;
-  return(ch);
+  return(ch ? ch : -1);
 }
 
 static inline signed char getch_peek(void) {
   signed char ch = (GLOBALS->vst_vcd_recoder_c_3!=GLOBALS->vend_vcd_recoder_c_3)?((int)(*GLOBALS->vst_vcd_recoder_c_3)):(getch_fetch());
   /* no increment */
-  return(ch);
+  return(ch ? ch : -1);
 }
 
 static int getch_patched(void)
@@ -957,7 +958,7 @@ return(T_UNKNOWN_KEY);
 }
 
 
-static int get_vartoken_patched(int match_kw)
+static int get_vartoken_patched(int ignore_colon, int match_kw)
 {
 int ch;
 int len=0;
@@ -979,7 +980,7 @@ if(!GLOBALS->var_prevch_vcd_recoder_c_3)
 	}
 
 if(ch=='[') return(V_LB);
-if(ch==':') return(V_COLON);
+if(ch==':' && !ignore_colon) return(V_COLON);
 if(ch==']') return(V_RB);
 
 for(GLOBALS->yytext_vcd_recoder_c_3[len++]=ch;;GLOBALS->yytext_vcd_recoder_c_3[len++]=ch)
@@ -990,7 +991,7 @@ for(GLOBALS->yytext_vcd_recoder_c_3[len++]=ch;;GLOBALS->yytext_vcd_recoder_c_3[l
 		}
 	ch=getch_patched();
 	if(ch<0) { free_2(GLOBALS->varsplit_vcd_recoder_c_3); GLOBALS->varsplit_vcd_recoder_c_3=NULL; break; }
-	if((ch==':')||(ch==']'))
+	if((ch==':' && !ignore_colon)||(ch==']'))
 		{
 		GLOBALS->var_prevch_vcd_recoder_c_3=ch;
 		break;
@@ -1013,14 +1014,14 @@ if(ch<0) { free_2(GLOBALS->varsplit_vcd_recoder_c_3); GLOBALS->varsplit_vcd_reco
 return(V_STRING);
 }
 
-static int get_vartoken(int match_kw)
+static int get_vartoken(int ignore_colon, int match_kw)
 {
 int ch;
 int len=0;
 
 if(GLOBALS->varsplit_vcd_recoder_c_3)
 	{
-	int rc=get_vartoken_patched(match_kw);
+	int rc=get_vartoken_patched(ignore_colon, match_kw);
 	if(rc!=V_END) return(rc);
 	GLOBALS->var_prevch_vcd_recoder_c_3=0;
 	}
@@ -1042,7 +1043,7 @@ if(!GLOBALS->var_prevch_vcd_recoder_c_3)
 	}
 
 if(ch=='[') return(V_LB);
-if(ch==':') return(V_COLON);
+if(ch==':' && !ignore_colon) return(V_COLON);
 if(ch==']') return(V_RB);
 
 if(ch=='#')	/* for MTI System Verilog '$var reg 64 >w #implicit-var###VarElem:ram_di[0.0] [63:0] $end' style declarations */
@@ -1054,7 +1055,16 @@ for(GLOBALS->yytext_vcd_recoder_c_3[len++]=ch;;GLOBALS->yytext_vcd_recoder_c_3[l
 	{
 	if(len==GLOBALS->T_MAX_STR_vcd_recoder_c_3)
 		{
-		GLOBALS->yytext_vcd_recoder_c_3=(char *)realloc_2(GLOBALS->yytext_vcd_recoder_c_3, (GLOBALS->T_MAX_STR_vcd_recoder_c_3=GLOBALS->T_MAX_STR_vcd_recoder_c_3*2)+1);
+		if(!GLOBALS->varsplit_vcd_recoder_c_3)
+			{
+			GLOBALS->yytext_vcd_recoder_c_3=(char *)realloc_2(GLOBALS->yytext_vcd_recoder_c_3, (GLOBALS->T_MAX_STR_vcd_recoder_c_3=GLOBALS->T_MAX_STR_vcd_recoder_c_3*2)+1);
+			}
+		else /* TALOS-2023-1806 */
+			{
+			int vsplit_len = GLOBALS->varsplit_vcd_recoder_c_3 - GLOBALS->yytext_vcd_recoder_c_3; /* save old len */
+			GLOBALS->yytext_vcd_recoder_c_3=(char *)realloc_2(GLOBALS->yytext_vcd_recoder_c_3, (GLOBALS->T_MAX_STR_vcd_recoder_c_3=GLOBALS->T_MAX_STR_vcd_recoder_c_3*2)+1);
+			GLOBALS->varsplit_vcd_recoder_c_3 = GLOBALS->yytext_vcd_recoder_c_3+vsplit_len; /* reconstruct old len in new buffer */
+			}
 		}
 
 	ch=getch();
@@ -1075,7 +1085,7 @@ for(GLOBALS->yytext_vcd_recoder_c_3[len++]=ch;;GLOBALS->yytext_vcd_recoder_c_3[l
 		GLOBALS->varsplit_vcd_recoder_c_3=GLOBALS->yytext_vcd_recoder_c_3+len;		/* keep looping so we get the *last* one */
 		}
 	else
-	if(((ch==':')||(ch==']'))&&(!GLOBALS->varsplit_vcd_recoder_c_3)&&(GLOBALS->yytext_vcd_recoder_c_3[0]!='\\'))
+	if(((ch==':' && !ignore_colon)||(ch==']'))&&(!GLOBALS->varsplit_vcd_recoder_c_3)&&(GLOBALS->yytext_vcd_recoder_c_3[0]!='\\'))
 		{
 		GLOBALS->var_prevch_vcd_recoder_c_3=ch;
 		break;
@@ -1179,6 +1189,11 @@ for(;;)
 		GLOBALS->autocoalesce = 0;
 		rc = 1;
 		}
+        else if(strstr(GLOBALS->yytext_vcd_recoder_c_3, "Questa") || strstr(GLOBALS->yytext_vcd_recoder_c_3, "ModelSim")) /* realparam fix only is for MTI, conflicts with Vivado */
+                {
+                GLOBALS->mti_realparam_fix = 1;
+                rc = 1;
+                }
 	}
 if(hdr) { DEBUG(fprintf(stderr,"\n")); }
 return(rc);
@@ -1289,6 +1304,15 @@ process_binary:
 				unsigned char typ2 = toupper(typ);
 				n->mv.mvlfac_vlist = (GLOBALS->vlist_prepack) ? ((struct vlist_t *)vlist_packer_create()) : vlist_create(sizeof(char));
 
+                                if((v->vartype == V_PARAMETER)&&(typ2=='R')) /* github #446: size 0 on parameter with type declared wrong */
+                                        {
+                                        v->vartype = V_REAL;    /* override any data we parsed in for $var declaration */
+                                        v->size=1;
+                                        v->msi=v->lsi=0;
+
+                                        fprintf(stderr, "GTKWAVE | Warning: symbol '%s' changing datatype from parameter to real.\n", v->name);
+                                        }
+
 				if((v->vartype!=V_REAL) && (v->vartype!=V_STRINGTYPE))
 					{
                                         if((typ2=='R')||(typ2=='S'))
@@ -1408,6 +1432,8 @@ static void vcd_parse(void)
 int tok;
 unsigned char ttype;
 int disable_autocoalesce = 0;
+int colon_seen = 0;
+int num_seen = 0;
 
 for(;;)
 	{
@@ -1587,7 +1613,7 @@ for(;;)
 			sync_end(NULL);
 			break;
 		case T_VAR:
-			if((GLOBALS->header_over_vcd_recoder_c_3)&&(0))
+			if(GLOBALS->header_over_vcd_recoder_c_3) /* reinstated because of TALOS-2023-1805 */
 			{
 			fprintf(stderr,"$VAR encountered after $ENDDEFINITIONS near byte %d.  VCD is malformed, exiting.\n",
 				(int)(GLOBALS->vcdbyteno_vcd_recoder_c_3+(GLOBALS->vst_vcd_recoder_c_3-GLOBALS->vcdbuf_vcd_recoder_c_3)));
@@ -1604,7 +1630,7 @@ for(;;)
                                 free_2(GLOBALS->varsplit_vcd_recoder_c_3);
                                 GLOBALS->varsplit_vcd_recoder_c_3=NULL;
                                 }
-			vtok=get_vartoken(1);
+			vtok=get_vartoken(0, 1);
 			if(vtok>V_STRINGTYPE) goto bail;
 
 			v=(struct vcdsymbol *)calloc_2(1,sizeof(struct vcdsymbol));
@@ -1613,7 +1639,7 @@ for(;;)
 
 			if(vtok==V_PORT)
 				{
-				vtok=get_vartoken(1);
+				vtok=get_vartoken(0, 1);
 				if(vtok==V_STRING)
 					{
 					v->size=atoi_64(GLOBALS->yytext_vcd_recoder_c_3);
@@ -1622,11 +1648,11 @@ for(;;)
 					else
 					if(vtok==V_LB)
 					{
-					vtok=get_vartoken(1);
+					vtok=get_vartoken(0, 1);
 					if(vtok==V_END) goto err;
 					if(vtok!=V_STRING) goto err;
 					v->msi=atoi_64(GLOBALS->yytext_vcd_recoder_c_3);
-					vtok=get_vartoken(0);
+					vtok=get_vartoken(0, 0);
 					if(vtok==V_RB)
 						{
 						v->lsi=v->msi;
@@ -1635,10 +1661,10 @@ for(;;)
 						else
 						{
 						if(vtok!=V_COLON) goto err;
-						vtok=get_vartoken(0);
+						vtok=get_vartoken(0, 0);
 						if(vtok!=V_STRING) goto err;
 						v->lsi=atoi_64(GLOBALS->yytext_vcd_recoder_c_3);
-						vtok=get_vartoken(0);
+						vtok=get_vartoken(0, 0);
 						if(vtok!=V_RB) goto err;
 
 						if(v->msi>v->lsi)
@@ -1676,7 +1702,7 @@ for(;;)
                                 if(v->nid < GLOBALS->vcd_minid_vcd_recoder_c_3) GLOBALS->vcd_minid_vcd_recoder_c_3 = v->nid;
                                 if(v->nid > GLOBALS->vcd_maxid_vcd_recoder_c_3) GLOBALS->vcd_maxid_vcd_recoder_c_3 = v->nid;
 
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(1, 0);
 				if(vtok!=V_STRING) goto err;
 				if(GLOBALS->slisthier_len)
 					{
@@ -1723,7 +1749,7 @@ for(;;)
 
                                 if(GLOBALS->pv_vcd_recoder_c_3)
                                         {
-                                        if(!strcmp(GLOBALS->prev_hier_uncompressed_name,v->name) && !disable_autocoalesce && (!strchr(v->name, '\\')))
+                                        if(!strcmp(GLOBALS->prev_hier_uncompressed_name,v->name) && !disable_autocoalesce && (!strchr(v->name, '\\')) && (v->lsi != GLOBALS->pv_vcd_recoder_c_3->lsi))
                                                 {
                                                 GLOBALS->pv_vcd_recoder_c_3->chain=v;
                                                 v->root=GLOBALS->rootv_vcd_recoder_c_3;
@@ -1746,7 +1772,7 @@ for(;;)
 				}
 				else	/* regular vcd var, not an evcd port var */
 				{
-				vtok=get_vartoken(1);
+				vtok=get_vartoken(0, 1);
 				if(vtok==V_END) goto err;
 				v->size=atoi_64(GLOBALS->yytext_vcd_recoder_c_3);
 				vtok=get_strtoken();
@@ -1772,7 +1798,7 @@ for(;;)
                                 if(v->nid < GLOBALS->vcd_minid_vcd_recoder_c_3) GLOBALS->vcd_minid_vcd_recoder_c_3 = v->nid;
                                 if(v->nid > GLOBALS->vcd_maxid_vcd_recoder_c_3) GLOBALS->vcd_maxid_vcd_recoder_c_3 = v->nid;
 
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(1, 0);
 				if(vtok!=V_STRING) goto err;
 
 				if(GLOBALS->slisthier_len)
@@ -1820,7 +1846,7 @@ for(;;)
 
                                 if(GLOBALS->pv_vcd_recoder_c_3)
                                         {
-                                        if(!strcmp(GLOBALS->prev_hier_uncompressed_name,v->name))
+                                        if(!strcmp(GLOBALS->prev_hier_uncompressed_name,v->name) && (v->lsi != GLOBALS->pv_vcd_recoder_c_3->lsi))
                                                 {
                                                 GLOBALS->pv_vcd_recoder_c_3->chain=v;
                                                 v->root=GLOBALS->rootv_vcd_recoder_c_3;
@@ -1840,23 +1866,27 @@ for(;;)
                                 GLOBALS->pv_vcd_recoder_c_3=v;
 				GLOBALS->prev_hier_uncompressed_name = strdup_2(v->name);
 
-				vtok=get_vartoken(1);
+				num_seen = 0;
+				vtok=get_vartoken(0, 1);
 				if(vtok==V_END) goto dumpv;
 				if(vtok!=V_LB) goto err;
-				vtok=get_vartoken(0);
+				colon_seen = 0;
+				vtok=get_vartoken(0, 0);
 				if(vtok!=V_STRING) goto err;
+				num_seen = 1;
 				v->msi=atoi_64(GLOBALS->yytext_vcd_recoder_c_3);
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(0, 0);
 				if(vtok==V_RB)
 					{
 					v->lsi=v->msi;
 					goto dumpv;
 					}
 				if(vtok!=V_COLON) goto err;
-				vtok=get_vartoken(0);
+				colon_seen = 1;
+				vtok=get_vartoken(0, 0);
 				if(vtok!=V_STRING) goto err;
 				v->lsi=atoi_64(GLOBALS->yytext_vcd_recoder_c_3);
-				vtok=get_vartoken(0);
+				vtok=get_vartoken(0, 0);
 				if(vtok!=V_RB) goto err;
 				}
 
@@ -1867,7 +1897,14 @@ for(;;)
 					{
 					if(v->vartype != V_STRINGTYPE)
 						{
-						v->vartype = V_REAL;
+						if(!GLOBALS->mti_realparam_fix)
+							{
+							v->size = 1;
+							}
+						else
+							{
+							v->vartype = V_REAL;
+							}
 						}
 					}
 					else
@@ -1876,7 +1913,47 @@ for(;;)
 					}
 
 				} /* MTI fix */
+				else
+				{
+                                unsigned int abslen = (v->msi >= v->lsi) ? (v->msi - v->lsi + 1) : (v->lsi - v->msi + 1);
 
+				if(!colon_seen)
+					{
+					if(num_seen && (v->size > 1))
+						{
+						char buf[32];
+						char *sfix;
+						int vname_len = strlen(v->name);
+						int num_len = snprintf(buf, 32, "[%d]", v->lsi);
+
+						sfix = malloc_2(vname_len + num_len + 1);
+						memcpy(sfix, v->name, vname_len);
+						memcpy(sfix+vname_len, buf, num_len + 1);
+						free_2(v->name);
+						v->name = sfix;
+						}
+
+					if(v->size > 1)
+						{
+						v->lsi = 0;
+						v->msi = v->size - 1;
+						}
+					}
+					else
+					{
+					if((v->size != abslen) && (num_seen))
+						{
+						if(v->lsi < v->msi)
+							{
+							v->msi = v->lsi + v->size - 1;
+							}
+							else
+							{
+							v->lsi = v->msi + v->size - 1;
+							}
+						}
+                                        }
+				}
 
 			if((v->vartype==V_REAL)||(v->vartype==V_STRINGTYPE))
 				{
@@ -2160,7 +2237,7 @@ for(;;)
 
 /*******************************************************************************/
 
-void add_histent(TimeType tim, struct Node *n, char ch, int regadd, char *vector)
+static void add_histent(TimeType tim, struct Node *n, char ch, int regadd, char *vector)
 {
 struct HistEnt *he;
 char heval;
@@ -2214,11 +2291,11 @@ if(!n->curr)
                 	he->v.h_val=heval;
 
                 	n->curr->next=he;
-			if(n->curr->v.h_val==heval)
-				{
-				n->curr->flags|=HIST_GLITCH;    /* set the glitch flag */
-				GLOBALS->num_glitch_regions_vcd_recoder_c_4++;				
-				}
+                        if(n->curr->v.h_val==heval)
+                                {
+                                n->curr->flags|=HIST_GLITCH;    /* set the glitch flag */
+                                GLOBALS->num_glitch_regions_vcd_recoder_c_4++;
+                                }
 			n->curr=he;
                 	GLOBALS->regions+=regadd;
 			}
@@ -2247,6 +2324,14 @@ switch(ch)
 		{
 		if(regadd) { tim*=(GLOBALS->time_scale); }
 
+                if(
+		  (n->curr->flags != (HIST_STRING|HIST_REAL)) /* because n->curr->v.h_vector could be a double and not a pointer */
+                  	||(n->curr->v.h_vector&&vector&&(strcmp(n->curr->v.h_vector,vector)))
+                        ||(tim==GLOBALS->start_time_vcd_recoder_c_3)
+                        ||(!n->curr->v.h_vector)
+                        ||(GLOBALS->vcd_preserve_glitches)
+                        ) /* same region == go skip */
+                        {
 			if(n->curr->time==tim)
 				{
 				DEBUG(printf("Warning: String Glitch at time ["TTFormat"] Signal [%p].\n",
@@ -2272,6 +2357,11 @@ switch(ch)
 				n->curr=he;
 	                	GLOBALS->regions+=regadd;
 				}
+                        }
+                        else
+                        {
+                        free_2(vector);
+                        }
 	       }
 	break;
 	}
@@ -2816,7 +2906,7 @@ if(suffix_check(fname, ".gz") || suffix_check(fname, ".zip"))
 	str=wave_alloca(strlen(fname)+dlen+1);
 	strcpy(str,WAVE_DECOMPRESSOR);
 	strcpy(str+dlen,fname);
-	GLOBALS->vcd_handle_vcd_recoder_c_2=popen(str,"r");
+	GLOBALS->vcd_handle_vcd_recoder_c_2=popen_san(str,"r");
 	GLOBALS->vcd_is_compressed_vcd_recoder_c_2=~0;
 	}
 	else
