@@ -15,6 +15,14 @@
 #include "hierpack.h"
 #include <time.h>
 
+/* can't use WAVE_NODEVARTYPE_STR because it has shorthand versions of various types which will fail to parse as VCD times with the VCD loaders */
+WAVE_NODEVARTYPE_VCD_SAVER_STR
+
+static unsigned int vartype_bounds_fix(unsigned int vt)
+{
+return( ((vt > ND_UNSPECIFIED_DEFAULT) && (vt <= ND_VARTYPE_MAX)) ? vt : ND_VCD_WIRE); /* GHW was printing "" for vartype */
+}
+
 static void w32redirect_fprintf(int is_trans, FILE *sfd, const char *format, ...)
 {
 #if defined _MSC_VER || defined __MINGW32__
@@ -291,6 +299,17 @@ if(vt->right) recurse_build(vt->right, hp);
 }
 
 
+static int vcdsav_tree_node_compare(const void *s1, const void *s2)
+{
+vcdsav_Tree *v1, *v2;
+
+v1=*((vcdsav_Tree **)s1);
+v2=*((vcdsav_Tree **)s2);
+
+return(v1->val - v2->val); /* emit in node add order seems good enough to maintain bitblasting order instead of something like (-sigcmp(v1->item->nname, v2->item->nname)) working directly on names */
+}
+
+
 /*
  * heapify algorithm...used to grab the next value change
  */
@@ -413,6 +432,7 @@ while(t)
 			if(!vt || vt->item != n)
 				{
 				unsigned char flags = 0;
+				unsigned char bitblasted = 0;
 
 				if(n->head.next)
 				if(n->head.next->next)
@@ -420,7 +440,47 @@ while(t)
 					flags = n->head.next->next->flags;
 					}
 
-				vt = vcdsav_insert(n, vt, ++nodecnt, flags, &n->head);
+				/* look to see if we need to emit all bits of a bitblasted vector in order for completeness */
+				if(n->msi == n->lsi) /* single bit */
+					{
+					char *lb = strrchr(n->nname, '[');
+					if(lb)
+						{
+						if(!strrchr(n->nname, ':')); /* look for bitblasting */
+							{
+							char *rb = strchr(lb+1, ']');
+							if(rb && !(*(rb+1))) /* ensure is at the end so it's not part of a generate index */
+								{
+								struct symbol *v = symfind(n->nname, NULL);
+								if(v->vec_root)
+									{
+									v = v->vec_root;
+									while(v)
+										{
+										vt = vcdsav_splay(v->n, vt);
+										if(!vt || vt->item != v->n)
+											{
+											bitblasted = 1;
+											flags = 0;
+											if(v->n->head.next)
+											if(v->n->head.next->next)
+												{
+												flags = v->n->head.next->next->flags;
+												}
+											vt = vcdsav_insert(v->n, vt, ++nodecnt, flags, &v->n->head);
+											}
+										v = v->vec_chain;
+										}
+									}
+								}
+							}
+						}
+					}
+
+				if(!bitblasted)
+					{
+					vt = vcdsav_insert(n, vt, ++nodecnt, flags, &n->head);
+					}
 				}
 			}
 		}
@@ -432,6 +492,7 @@ while(t)
 			bptr bt = b->bits;
 			if(bt)
 				{
+				struct symbol *vr = NULL; /* used to keep from having to go O(n**2) in worst case across bt->nnbits when bitblasted */
 				for(i=0;i<bt->nnbits;i++)
 					{
 					if(bt->nodes[i])
@@ -443,6 +504,7 @@ while(t)
 						if(!vt || vt->item != n)
 							{
 							unsigned char flags = 0;
+							unsigned char bitblasted = 0;
 
 							if(n->head.next)
 							if(n->head.next->next)
@@ -450,7 +512,49 @@ while(t)
 								flags = n->head.next->next->flags;
 								}
 
-							vt = vcdsav_insert(n, vt, ++nodecnt, flags, &n->head);
+							/* look to see if we need to emit all bits of a bitblasted vector in order for completeness */
+							if(n->msi == n->lsi) /* single bit */
+								{
+								char *lb = strrchr(n->nname, '[');
+								if(lb)
+									{
+									if(!strrchr(n->nname, ':')); /* look for bitblasting */
+										{
+			                                                        char *rb = strchr(lb+1, ']');
+                        			                                if(rb && !(*(rb+1))) /*	ensure is at the end so	it's not part of a generate index */
+                                                			                {
+											struct symbol *v = symfind(n->nname, NULL);
+											if((v->vec_root) && (v->vec_root != vr))
+												{
+												v = vr = v->vec_root;
+												while(v)
+													{
+													vt = vcdsav_splay(v->n, vt);
+													if(!vt || vt->item != v->n)
+														{
+														bitblasted = 1;
+														flags = 0;
+
+														if(v->n->head.next)
+														if(v->n->head.next->next)
+															{
+															flags = v->n->head.next->next->flags;
+															}
+
+														vt = vcdsav_insert(v->n, vt, ++nodecnt, flags, &v->n->head);
+														}
+													v = v->vec_chain;
+													}
+												}
+											}
+										}
+									}
+								}
+
+							if(!bitblasted)
+								{
+								vt = vcdsav_insert(n, vt, ++nodecnt, flags, &n->head);
+								}
 							}
 						}
 					}
@@ -560,6 +664,7 @@ if(export_typ == WAVE_EXPORT_TRANS)
 /* write out netnames here ... */
 hp_clone = GLOBALS->hp_vcd_saver_c_1 = calloc_2(nodecnt, sizeof(vcdsav_Tree *));
 recurse_build(vt, &hp_clone);
+qsort(GLOBALS->hp_vcd_saver_c_1, nodecnt, sizeof(vcdsav_Tree *), vcdsav_tree_node_compare); /* reconstruct array in order vcdsav_insert() was called to get bit ordering back */
 
 for(i=0;i<nodecnt;i++)
 	{
@@ -580,7 +685,7 @@ for(i=0;i<nodecnt;i++)
 			}
 			else
 			{
-			const char *typ = (GLOBALS->hp_vcd_saver_c_1[i]->flags & HIST_STRING) ? "string" : "real";
+			const char *typ = (GLOBALS->hp_vcd_saver_c_1[i]->flags & HIST_STRING) ? vartype_vcd_saver_strings[ND_GEN_STRING] : vartype_vcd_saver_strings[ND_VCD_REAL];
 			int tlen = (GLOBALS->hp_vcd_saver_c_1[i]->flags & HIST_STRING) ? 0 : 1;
 			w32redirect_fprintf(is_trans, GLOBALS->f_vcd_saver_c_1, "$var %s %d %s %s $end\n", typ, tlen, vcdid(GLOBALS->hp_vcd_saver_c_1[i]->val, export_typ), netname);
 			}
@@ -608,7 +713,7 @@ for(i=0;i<nodecnt;i++)
 				}
 				else
 				{
-				w32redirect_fprintf(is_trans, GLOBALS->f_vcd_saver_c_1, "$var wire 1 %s %s $end\n", vcdid(GLOBALS->hp_vcd_saver_c_1[i]->val, export_typ), netname);
+				w32redirect_fprintf(is_trans, GLOBALS->f_vcd_saver_c_1, "$var %s 1 %s %s $end\n", vartype_vcd_saver_strings[vartype_bounds_fix(GLOBALS->hp_vcd_saver_c_1[i]->item->vartype)], vcdid(GLOBALS->hp_vcd_saver_c_1[i]->val, export_typ), netname);
 				}
 			}
 			else
@@ -620,7 +725,7 @@ for(i=0;i<nodecnt;i++)
 				}
 				else
 				{
-				w32redirect_fprintf(is_trans, GLOBALS->f_vcd_saver_c_1, "$var wire %d %s %s $end\n", len, vcdid(GLOBALS->hp_vcd_saver_c_1[i]->val, export_typ), netname);
+				w32redirect_fprintf(is_trans, GLOBALS->f_vcd_saver_c_1, "$var %s %d %s %s $end\n", vartype_vcd_saver_strings[vartype_bounds_fix(GLOBALS->hp_vcd_saver_c_1[i]->item->vartype)], len, vcdid(GLOBALS->hp_vcd_saver_c_1[i]->val, export_typ), netname);
 				}
 			GLOBALS->hp_vcd_saver_c_1[i]->len = len;
 			if(len > max_len) max_len = len;
